@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <KLocalizedString>
 #include <QDebug>
 #if QT_VERSION < 0x050500
 # define qInfo qDebug
@@ -27,6 +28,7 @@
 #include <QCursor>
 #include <QMouseEvent>
 #include <QTimer>
+#include <QMap>
 #include <vlc/vlc.h>
 
 #include "../configuration.h"
@@ -56,30 +58,30 @@ bool VlcMediaWidget::init()
 
 	vlcInstance = libvlc_new(argc, argv);
 	if (!vlcInstance) {
-		qInfo() << "libVLC: failed to use extra args:" << args;
+		qWarning("libVLC: failed to use extra args: %s", qPrintable(args));
 		argc = 0;
 		vlcInstance = libvlc_new(0, NULL);
 		if (vlcInstance)
-			qInfo() << "Using libVLC without arguments";
+			qInfo("Using libVLC without arguments");
 	}
 
 	if (vlcInstance == NULL) {
-		qInfo() << "VlcMediaWidget::init: cannot create vlc instance" << QLatin1String(libvlc_errmsg());
+		qFatal("Cannot create vlc instance %s", qPrintable(libvlc_errmsg()));
 		return false;
 	}
 
 	if (argc) {
-		QString log = QLatin1String("Using libVLC with ");
+		QString log = "Using libVLC with args:";
 		for (int i = 0; i < argc; i++)
 			log += " " + QLatin1String(argv[i]);
 
-		qInfo() << log;
+		qDebug("%s", qPrintable(log));
 	}
 
 	vlcMediaPlayer = libvlc_media_player_new(vlcInstance);
 
 	if (vlcMediaPlayer == NULL) {
-		qInfo() << "VlcMediaWidget::init: cannot create vlc media player" << QLatin1String(libvlc_errmsg());
+		qFatal("Cannot create vlc media player %s", qPrintable(libvlc_errmsg()));
 		return false;
 	}
 
@@ -91,7 +93,7 @@ bool VlcMediaWidget::init()
 
 	for (uint i = 0; i < (sizeof(eventTypes) / sizeof(eventTypes[0])); ++i) {
 		if (libvlc_event_attach(eventManager, eventTypes[i], vlcEventHandler, this) != 0) {
-			qInfo() << "VlcMediaWidget::init: cannot attach event handler" << eventTypes[i];
+			qCritical("Cannot attach event handler %s", qPrintable(eventTypes[i]));
 			return false;
 		}
 	}
@@ -130,6 +132,43 @@ VlcMediaWidget *VlcMediaWidget::createVlcMediaWidget(QWidget *parent)
 	return vlcMediaWidget.take();
 }
 
+QStringList VlcMediaWidget::getAudioDevices()
+{
+	libvlc_audio_output_device_t *vlcAudioOutput, *i;
+	QStringList audioDevices;
+
+	// Get audio device list
+	vlcAudioOutput = libvlc_audio_output_device_enum(vlcMediaPlayer);
+	if (!vlcAudioOutput)
+		return audioDevices;
+
+	for (i = vlcAudioOutput; i != NULL; i = i->p_next) {
+		QString device = QString::fromUtf8(i->psz_description);
+		audioDevices.append(device);
+	}
+	libvlc_audio_output_device_list_release(vlcAudioOutput);
+
+	return audioDevices;
+}
+
+void VlcMediaWidget::setAudioDevice(QString device)
+{
+	libvlc_audio_output_device_t *vlcAudioOutput, *i;
+	vlcAudioOutput = libvlc_audio_output_device_enum(vlcMediaPlayer);
+
+	if (!vlcAudioOutput)
+		return;
+
+	for (i = vlcAudioOutput; i != NULL; i = i->p_next) {
+		if (device.compare(QString::fromUtf8(i->psz_description)))
+			continue;
+		qDebug("Setting audio output to: %s", qPrintable(i->psz_device));
+
+		libvlc_audio_output_device_set(vlcMediaPlayer, NULL, i->psz_device);
+	}
+	libvlc_audio_output_device_list_release(vlcAudioOutput);
+}
+
 void VlcMediaWidget::setMuted(bool muted)
 {
 	libvlc_audio_set_mute(vlcMediaPlayer, muted);
@@ -139,7 +178,7 @@ void VlcMediaWidget::setVolume(int volume)
 {
 	// 0 <= volume <= 200
 	if (libvlc_audio_set_volume(vlcMediaPlayer, volume) != 0) {
-		qInfo() << "VlcMediaWidget::setVolume: cannot set volume" << volume;
+		qWarning("cannot set volume %i", volume);
 	}
 }
 
@@ -227,7 +266,7 @@ void VlcMediaWidget::play(const MediaSource &source)
 
 	if (vlcMedia == NULL) {
 		libvlc_media_player_stop(vlcMediaPlayer);
-		qInfo() << "VlcMediaWidget::play: cannot create media" << source.getUrl().toDisplayString();
+		qWarning("Cannot create media %s", qPrintable(source.getUrl().toDisplayString()));
 		return;
 	}
 
@@ -236,25 +275,19 @@ void VlcMediaWidget::play(const MediaSource &source)
 
 	for (uint i = 0; i < (sizeof(eventTypes) / sizeof(eventTypes[0])); ++i) {
 		if (libvlc_event_attach(eventManager, eventTypes[i], vlcEventHandler, this) != 0) {
-			qInfo() << "VlcMediaWidget::play: cannot attach event handler" << eventTypes[i];
+			qWarning("Cannot attach event handler %s", qPrintable(eventTypes[i]));
 		}
 	}
 
 	libvlc_media_player_set_media(vlcMediaPlayer, vlcMedia);
 	libvlc_media_release(vlcMedia);
 
-//	FIXME!
-
-// 	if (source.subtitleUrl.isValid()) {
-// 		if (libvlc_video_set_subtitle_file(vlcMediaPlayer,
-// 		    source.subtitleUrl.toEncoded().constData()) == 0) {
-// 			qInfo() << "VlcMediaWidget::play: cannot set subtitle file" <<
-// 				source.subtitleUrl.toDisplayString();
-// 		}
-// 	}
+//	FIXME! subtitleUrl is only available for MediaSourceUrl private class
+//	if (source.subtitleUrl.isValid())
+//		setExternalSubtitle(source.subtitleUrl);
 
 	if (libvlc_media_player_play(vlcMediaPlayer) != 0) {
-		qInfo() << "VlcMediaWidget::play: cannot play media" << source.getUrl().toDisplayString();
+		qWarning("Cannot play media %s", qPrintable(source.getUrl().toDisplayString()));
 	}
 
 	setCursor(Qt::BlankCursor);
@@ -293,22 +326,15 @@ void VlcMediaWidget::setCurrentAudioStream(int currentAudioStream)
 
 void VlcMediaWidget::setCurrentSubtitle(int currentSubtitle)
 {
-	int requestedSubtitle = currentSubtitle;
-	int availableSpuCount = libvlc_video_get_spu_count(vlcMediaPlayer);
+	int requestedSubtitle = -1;
 
-	if (requestedSubtitle >= availableSpuCount) {
-		qInfo() << "VlcMediaWidget::setCurrentSubtitle: subtitle is out of range" <<
-			requestedSubtitle << availableSpuCount;
-
-		// Disable subtitles.
-		requestedSubtitle = -1;
-	} else if (requestedSubtitle < 0) {
-		// Set all negative subtitle requests to -1 as this makes libvlc
-		// disable the subtitles.
-		requestedSubtitle = -1;
-	} else {
-		// We got a valid subtitle request - skip the 'deactivate' subtitle.
-		requestedSubtitle += 1;
+	QMap<int, int>::const_iterator i = subtitleId.constBegin();
+	while (i != subtitleId.constEnd()) {
+		if (i.value() == currentSubtitle) {
+			requestedSubtitle = i.key();
+			break;
+		}
+		i++;
 	}
 
 	libvlc_video_set_spu(vlcMediaPlayer, requestedSubtitle);
@@ -318,8 +344,7 @@ void VlcMediaWidget::setExternalSubtitle(const QUrl &subtitleUrl)
 {
 	if (libvlc_video_set_subtitle_file(vlcMediaPlayer,
 	    subtitleUrl.toEncoded().constData()) == 0) {
-		qInfo() << "VlcMediaWidget::setExternalSubtitle: cannot set subtitle file" <<
-			subtitleUrl.toDisplayString();
+		qWarning("Cannot set subtitle file %s", qPrintable(subtitleUrl.toDisplayString()));
 	}
 }
 
@@ -490,7 +515,9 @@ void VlcMediaWidget::updateSubtitles()
 {
 	subtitles.clear();
 	libvlc_track_description_t *track = libvlc_video_get_spu_description(vlcMediaPlayer);
+	int i = 0;
 
+	subtitleId.clear();
 	if (track != NULL) {
 		// skip the 'deactivate' subtitle
 		track = track->p_next;
@@ -498,27 +525,22 @@ void VlcMediaWidget::updateSubtitles()
 
 	while (track != NULL) {
 		QString subtitle = QString::fromUtf8(track->psz_name);
-		int cutBegin = (subtitle.indexOf(QLatin1Char('[')) + 1);
-
-		if (cutBegin > 0) {
-			int cutEnd = subtitle.lastIndexOf(QLatin1Char(']'));
-
-			if (cutEnd >= 0) {
-				// remove unnecessary text
-				subtitle = subtitle.mid(cutBegin, cutEnd - cutBegin);
-			}
-		}
 
 		if (subtitle.isEmpty()) {
-			subtitle = QString::number(subtitles.size() + 1);
+			subtitle = i18n("Subtitle %1", track->i_id);
 		}
 
+		// 0 is reserved for "disabled" at mediawidget. So, we should
+		// Start counting from 1, to match the range expected for
+		// currentSubtitle
+		subtitleId[track->i_id] = ++i;
 		subtitles.append(subtitle);
 		track = track->p_next;
 	}
+	libvlc_track_description_list_release(track);
 
 	// skip the 'deactivate' subtitle
-	currentSubtitle = (libvlc_video_get_spu(vlcMediaPlayer) - 1);
+	currentSubtitle = subtitleId.value(libvlc_video_get_spu(vlcMediaPlayer), -1);
 }
 
 void VlcMediaWidget::updateTitles()
@@ -587,7 +609,7 @@ void VlcMediaWidget::vlcEvent(const libvlc_event_t *event)
 
 	switch (event->type) {
 	case libvlc_MediaMetaChanged:
-		pendingUpdatesToBeAdded = Metadata;
+		pendingUpdatesToBeAdded = Metadata | Subtitles;
 		break;
 	case libvlc_MediaPlayerEncounteredError:
 		pendingUpdatesToBeAdded = PlaybackStatus;
@@ -599,7 +621,7 @@ void VlcMediaWidget::vlcEvent(const libvlc_event_t *event)
 		pendingUpdatesToBeAdded = CurrentTotalTime;
 		break;
 	case libvlc_MediaPlayerSeekableChanged:
-		pendingUpdatesToBeAdded = Seekable;
+		pendingUpdatesToBeAdded = Seekable | Subtitles;
 		break;
 	case libvlc_MediaPlayerStopped:
 		pendingUpdatesToBeAdded = PlaybackStatus;
@@ -613,7 +635,7 @@ void VlcMediaWidget::vlcEvent(const libvlc_event_t *event)
 	if (pendingUpdatesToBeAdded != 0) {
 		addPendingUpdates(pendingUpdatesToBeAdded);
 	} else {
-		qInfo() << "VlcMediaWidget::vlcEvent: unknown event type" << event->type;
+		qWarning("Unknown libVLC event type %d", event->type);
 	}
 }
 
