@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <KLocalizedString>
 #include <QDebug>
 #if QT_VERSION < 0x050500
 # define qInfo qDebug
@@ -69,7 +70,7 @@ void DvbLinuxCam::startCa(const QString &path)
 	caFd = open(QFile::encodeName(path).constData(), O_RDWR | O_NONBLOCK);
 
 	if (caFd < 0) {
-		qInfo() << "DvbLinuxCam::startCa: cannot open" << path;
+		qWarning("Cannot open CA device node %s", qPrintable(path));
 		return;
 	}
 
@@ -87,7 +88,7 @@ void DvbLinuxCam::startDescrambling(const QByteArray &pmtSectionData)
 	DvbPmtSection pmtSection(pmtSectionData);
 
 	if (!pmtSection.isValid()) {
-		qInfo() << "DvbLinuxCam::startDescrambling: pmt section is invalid";
+		qWarning("PMT section is invalid while descrambling");
 		return;
 	}
 
@@ -115,7 +116,7 @@ void DvbLinuxCam::stopDescrambling(int serviceId)
 	QMap<int, DvbLinuxCamService>::iterator it = services.find(serviceId);
 
 	if (it == services.end()) {
-		qInfo() << "DvbLinuxCam::stopDescrambling: cannot find service id" << serviceId;
+		qWarning("Cannot find service id %d while stopping CAM", serviceId);
 		return;
 	}
 
@@ -128,7 +129,7 @@ void DvbLinuxCam::stopDescrambling(int serviceId)
 		services.erase(it);
 		return;
 	case DvbLinuxCamService::Remove:
-		qInfo() << "DvbLinuxCam::stopDescrambling: service is already being removed";
+		qWarning("CAM Service was already removed");
 		services.erase(it);
 		return;
 	}
@@ -163,7 +164,7 @@ void DvbLinuxCam::pollModule()
 	} else {
 		if ((pendingCommands & ExpectingReply) != 0) {
 			pendingCommands &= ~ExpectingReply;
-			qInfo() << "DvbLinuxCam::pollModule: request timed out";
+			qDebug("CAM: request timed out");
 		}
 
 		if (pendingCommands == 0) {
@@ -207,7 +208,7 @@ void DvbLinuxCam::readyRead()
 		handleTransportLayer(data + 2, size - 2);
 		handlePendingCommands();
 	} else {
-		qInfo() << "DvbLinuxCam::readyRead: unknown recipient";
+		qWarning("CAM: unknown recipient");
 	}
 }
 
@@ -218,7 +219,7 @@ bool DvbLinuxCam::detectSlot()
 	memset(&caInfo, 0, sizeof(caInfo));
 
 	if (ioctl(caFd, CA_GET_CAP, &caInfo) != 0) {
-		qInfo() << "DvbLinuxCam::detectSlot: cannot perform ioctl CA_GET_CAP";
+		qWarning("Cannot perform ioctl CA_GET_CAP");
 		return false;
 	}
 
@@ -228,13 +229,12 @@ bool DvbLinuxCam::detectSlot()
 		slotInfo.num = i;
 
 		if (ioctl(caFd, CA_GET_SLOT_INFO, &slotInfo) != 0) {
-			qInfo() << "DvbLinuxCam::detectSlot: "
-			    "cannot perform ioctl CA_GET_SLOT_INFO for slot" << slot;
+			qWarning("Cannot perform ioctl CA_GET_SLOT_INFO for slot %d", slot);
 			continue;
 		}
 
 		if ((slotInfo.type & CA_CI_LINK) == 0) {
-			qInfo() << "DvbLinuxCam::detectSlot: unknown type" << slotInfo.type;
+			qWarning("Unknown CAM CI link type %d", slotInfo.type);
 			continue;
 		}
 
@@ -315,8 +315,7 @@ void DvbLinuxCam::handleTransportLayer(const unsigned char *data, int size)
 		case StatusByte:
 			if ((length < 2) || (data[0] != ConnectionId)) {
 				size = 0;
-				qInfo() << "DvbLinuxCam::handleTransportLayer: "
-				    "invalid StatusByte object";
+				qWarning("CAM: invalid StatusByte object");
 				break;
 			}
 
@@ -328,8 +327,7 @@ void DvbLinuxCam::handleTransportLayer(const unsigned char *data, int size)
 		case CreateTransportConnectionReply:
 			if ((length < 1) || (data[0] != ConnectionId)) {
 				size = 0;
-				qInfo() << "DvbLinuxCam::handleTransportLayer: "
-				    "invalid CreateTransportConnectionReply object";
+				qWarning("CAM: invalid CreateTransportConnectionReply object");
 				break;
 			}
 
@@ -338,14 +336,14 @@ void DvbLinuxCam::handleTransportLayer(const unsigned char *data, int size)
 		case DataLast:
 			if ((length < 1) || (data[0] != ConnectionId)) {
 				size = 0;
-				qInfo() << "DvbLinuxCam::handleTransportLayer: invalid DataLast object";
+				qWarning("CAM: invalid DataLast object");
 				break;
 			}
 
 			handleSessionLayer(data + 1, length - 1);
 			break;
 		default:
-			qInfo() << "DvbLinuxCam::handleTransportLayer: unknown tag" << tag;
+			qWarning("CAM: unknown tag %d", tag);
 			break;
 		}
 
@@ -365,8 +363,7 @@ void DvbLinuxCam::handleSessionLayer(const unsigned char *data, int size)
 		switch (tag) {
 		case OpenSessionRequest: {
 			if (length < 4) {
-				qInfo() << "DvbLinuxCam::handleSessionLayer: "
-				    "invalid OpenSessionRequest object";
+				qWarning("CAM: invalid OpenSessionRequest object");
 				break;
 			}
 
@@ -404,15 +401,14 @@ void DvbLinuxCam::handleSessionLayer(const unsigned char *data, int size)
 		    }
 		case SessionNumber:
 			if (length < 2) {
-				qInfo() << "DvbLinuxCam::handleSessionLayer: "
-				    "invalid SessionNumber object";
+				qWarning("CAM: invalid SessionNumber object");
 				break;
 			}
 
 			handleApplicationLayer(data + length, size - length);
 			break;
 		default:
-			qInfo() << "DvbLinuxCam::handleSessionLayer: unknown tag" << tag;
+			qWarning("CAM: unknown tag %d", tag);
 			break;
 		}
 	}
@@ -453,7 +449,7 @@ void DvbLinuxCam::handleApplicationLayer(const unsigned char *data, int size)
 			QCoreApplication::postEvent(this, new QEvent(QEvent::User));
 			break;
 		default:
-			qInfo() << "DvbLinuxCam::handleApplicationLayer: unknown tag" << tag;
+			qWarning("CAM: unknown tag %d", tag);
 			break;
 		}
 
@@ -473,11 +469,10 @@ void DvbLinuxCam::handlePendingCommands()
 			break;
 		case ResetCa:
 			if (ioctl(caFd, CA_RESET, 0xff) != 0) {
-				qInfo() << "DvbLinuxCam::handlePendingCommands: "
-				    "cannot perform ioctl CA_RESET";
+				qWarning("Cannot perform ioctl CA_RESET");
 			}
 
-			qInfo() << "DvbLinuxCam::handlePendingCommands: --> reset";
+			qDebug("--> CAM reset");
 			slot = -1;
 			pollTimer.start(100);
 			pendingCommands = 0;
@@ -510,8 +505,7 @@ void DvbLinuxCam::handlePendingCommands()
 			sendApplicationLayerMessage(CaInfoEnquiry, messageData, messageData);
 			break;
 		default:
-			qInfo() << "DvbLinuxCam::handlePendingCommands: unknown pending command" <<
-				pendingCommand;
+			qWarning("CAM: unknown pending command %d", pendingCommand);
 			break;
 		}
 	}
@@ -560,7 +554,7 @@ void DvbLinuxCam::customEvent(QEvent *event)
 			break;
 		    }
 		case DvbLinuxCamService::Remove:
-			qInfo() << "DvbLinuxCam::customEvent: impossible";
+			qWarning("CAM: impossible to remove custom event");
 			break;
 		}
 
@@ -654,8 +648,7 @@ void DvbLinuxCam::sendTransportLayerMessage(TransportLayerTag tag, char *data, c
 	length = uint(end - data);
 
 	if (write(caFd, data, length) != length) {
-		qInfo() << "DvbLinuxCam::sendTransportLayerMessage: cannot send message of length" <<
-			length;
+		qWarning("CAM: cannot send message of length %d", length);
 	}
 
 	pendingCommands |= ExpectingReply;
