@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <KLocalizedString>
 #include <QDebug>
 #if QT_VERSION < 0x050500
 # define qInfo qDebug
@@ -25,6 +26,7 @@
 
 #include <QDataStream>
 #include <QFile>
+#include <QLoggingCategory>
 #include <QStandardPaths>
 
 #include "../ensurenopendingoperation.h"
@@ -94,21 +96,23 @@ DvbEpgModel::DvbEpgModel(DvbManager *manager_, QObject *parent) : QObject(parent
 	QFile file(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1String("/epgdata.dvb"));
 
 	if (!file.open(QIODevice::ReadOnly)) {
-		qInfo() << "DvbEpgModel::DvbEpgModel: cannot open" << file.fileName();
+		qWarning("Cannot open %s", qPrintable(file.fileName()));
 		return;
 	}
 
 	QDataStream stream(&file);
 	stream.setVersion(QDataStream::Qt_4_4);
 	DvbRecordingModel *recordingModel = manager->getRecordingModel();
-	bool hasRecordingKey = true;
+	bool hasRecordingKey = true, hasParental = true;
 	int version;
 	stream >> version;
 
 	if (version == 0x1ce0eca7) {
 		hasRecordingKey = false;
-	} else if (version != 0x79cffd36) {
-		qInfo() << "DvbEpgModel::DvbEpgModel: wrong version" << file.fileName();
+	} else if (version == 0x79cffd36) {
+		hasParental = false;
+	} else if (version != 0x140c37b5) {
+		qWarning("Wrong DB version for: %s", qPrintable(file.fileName()));
 		return;
 	}
 
@@ -133,8 +137,21 @@ DvbEpgModel::DvbEpgModel(DvbManager *manager_, QObject *parent) : QObject(parent
 			}
 		}
 
+		if (hasParental) {
+			unsigned tmp;
+
+			stream >> tmp;
+			stream >> entry.content;
+			stream >> entry.parental;
+
+			if (tmp <= DvbEpgEntry::EitLast)
+				entry.type = DvbEpgEntry::EitType(tmp);
+			else
+				entry.type = DvbEpgEntry::EitActualTsSchedule;
+		}
+
 		if (stream.status() != QDataStream::Ok) {
-			qInfo() << "DvbEpgModel::DvbEpgModel: corrupt data" << file.fileName();
+			qWarning("Corrupt data %s", qPrintable(file.fileName()));
 			break;
 		}
 
@@ -145,23 +162,23 @@ DvbEpgModel::DvbEpgModel(DvbManager *manager_, QObject *parent) : QObject(parent
 DvbEpgModel::~DvbEpgModel()
 {
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::~DvbEpgModel: illegal recursive call";
+		qWarning("Illegal recursive call");
 	}
 
 	if (!dvbEpgFilters.isEmpty() || !atscEpgFilters.isEmpty()) {
-		qInfo() << "DvbEpgModel::~DvbEpgModel: filter list not empty";
+		qWarning("EPG filter list not empty");
 	}
 
 	QFile file(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1String("/epgdata.dvb"));
 
 	if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-		qInfo() << "DvbEpgModel::~DvbEpgModel: cannot open" << file.fileName();
+		qWarning("Cannot open %s", qPrintable(file.fileName()));
 		return;
 	}
 
 	QDataStream stream(&file);
 	stream.setVersion(QDataStream::Qt_4_4);
-	int version = 0x79cffd36;
+	int version = 0x140c37b5;
 	stream << version;
 
 	foreach (const DvbSharedEpgEntry &entry, entries) {
@@ -178,6 +195,9 @@ DvbEpgModel::~DvbEpgModel()
 		stream << entry->subheading;
 		stream << entry->details;
 		stream << recordingKey.sqlKey;
+		stream << int(entry->type);
+		stream << entry->content;
+		stream << entry->parental;
 	}
 }
 
@@ -224,20 +244,65 @@ QList<DvbSharedEpgEntry> DvbEpgModel::getCurrentNext(const DvbSharedChannel &cha
 	return result;
 }
 
+void DvbEpgModel::Debug(QString text, const DvbSharedEpgEntry &entry)
+{
+	if (!QLoggingCategory::defaultCategory()->isEnabled(QtDebugMsg))
+		return;
+
+	QDateTime begin = entry->begin.toLocalTime();
+	QTime end = entry->begin.addSecs(QTime(0, 0, 0).secsTo(entry->duration)).toLocalTime().time();
+
+	qDebug("EPG event %s: type %d, from %s to %s: %s: %s: %s : %s", qPrintable(text), entry->type, qPrintable(QLocale().toString(begin, QLocale::ShortFormat)), qPrintable(QLocale().toString(end)), qPrintable(entry->title), qPrintable(entry->subheading), qPrintable(entry->details), qPrintable(entry->content));
+}
+
 DvbSharedEpgEntry DvbEpgModel::addEntry(const DvbEpgEntry &entry)
 {
 	if (!entry.validate()) {
-		qInfo() << "DvbEpgModel::addEntry: invalid entry";
+		qWarning("Invalid entry: channel is %s, begin is %s, duration is %s", entry.channel.isValid() ? "valid" : "invalid", entry.begin.isValid() ? "valid" : "invalid", entry.duration.isValid() ? "valid" : "invalid");
 		return DvbSharedEpgEntry();
 	}
 
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::addEntry: illegal recursive call";
+		qWarning("Iillegal recursive call");
 		return DvbSharedEpgEntry();
 	}
 
 	EnsureNoPendingOperation ensureNoPendingOperation(hasPendingOperation);
 
+	// Check if the event was already recorded
+	const QDateTime end = entry.begin.addSecs(QTime(0, 0, 0).secsTo(entry.duration));
+
+	for (Iterator it = entries.begin(); ConstIterator(it) != entries.constEnd(); it++) {
+		const DvbSharedEpgEntry &existingEntry = *it;
+
+		// Don't do anything if the event already exists
+		if (*existingEntry == entry)
+			return DvbSharedEpgEntry();
+
+		if (existingEntry->channel != entry.channel)
+			continue;
+
+		const QDateTime enEnd = existingEntry->begin.addSecs(QTime(0, 0, 0).secsTo(existingEntry->duration));
+
+		// A new event conflicts with an existing one
+		if (((entry.begin > existingEntry->begin) && (entry.begin < enEnd)) || ((end > existingEntry->begin) && end < enEnd)) {
+			Debug("removed", existingEntry);
+			it = removeEntry(it);
+			break;
+		}
+		// New event data for the same event
+		if (((entry.begin == existingEntry->begin) && (end == enEnd))) {
+			if (existingEntry->details.isEmpty() && !entry.details.isEmpty()) {
+				emit entryAboutToBeUpdated(existingEntry);
+				const_cast<DvbEpgEntry *>(existingEntry.constData())->details =
+					entry.details;
+				emit entryUpdated(existingEntry);
+				Debug("updated", existingEntry);
+			}
+			return existingEntry;
+		}
+	}
+
 	if (entry.begin.addSecs(QTime(0, 0, 0).secsTo(entry.duration)) > currentDateTimeUtc) {
 		DvbSharedEpgEntry existingEntry = entries.value(DvbEpgEntryId(&entry));
 
@@ -248,6 +313,7 @@ DvbSharedEpgEntry DvbEpgModel::addEntry(const DvbEpgEntry &entry)
 				const_cast<DvbEpgEntry *>(existingEntry.constData())->details =
 					entry.details;
 				emit entryUpdated(existingEntry);
+				Debug("updated2", existingEntry);
 			}
 
 			return existingEntry;
@@ -265,6 +331,7 @@ DvbSharedEpgEntry DvbEpgModel::addEntry(const DvbEpgEntry &entry)
 		}
 
 		emit entryAdded(newEntry);
+		Debug("new", newEntry);
 		return newEntry;
 	}
 
@@ -275,12 +342,12 @@ void DvbEpgModel::scheduleProgram(const DvbSharedEpgEntry &entry, int extraSecon
 	int extraSecondsAfter, bool checkForRecursion, int priority)
 {
 	if (!entry.isValid() || (entries.value(DvbEpgEntryId(entry)) != entry)) {
-		qInfo() << "DvbEpgModel::scheduleProgram: invalid entry";
+		qWarning("Can't schedule program: invalid entry");
 		return;
 	}
 
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::scheduleProgram: illegal recursive call";
+		qWarning("Illegal recursive call");
 		return;
 	}
 
@@ -389,7 +456,7 @@ void DvbEpgModel::channelAboutToBeUpdated(const DvbSharedChannel &channel)
 void DvbEpgModel::channelUpdated(const DvbSharedChannel &channel)
 {
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::channelUpdated: illegal recursive call";
+		qWarning("Illegal recursive call");
 		return;
 	}
 
@@ -408,7 +475,7 @@ void DvbEpgModel::channelUpdated(const DvbSharedChannel &channel)
 void DvbEpgModel::channelRemoved(const DvbSharedChannel &channel)
 {
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::channelRemoved: illegal recursive call";
+		qWarning("Illegal recursive call");
 		return;
 	}
 
@@ -424,7 +491,7 @@ void DvbEpgModel::channelRemoved(const DvbSharedChannel &channel)
 void DvbEpgModel::recordingRemoved(const DvbSharedRecording &recording)
 {
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::recordingRemoved: illegal recursive call";
+		qWarning("Illegal recursive call");
 		return;
 	}
 
@@ -443,7 +510,7 @@ void DvbEpgModel::timerEvent(QTimerEvent *event)
 	Q_UNUSED(event)
 
 	if (hasPendingOperation) {
-		qInfo() << "DvbEpgModel::timerEvent: illegal recursive call";
+		qWarning("Illegal recursive call");
 		return;
 	}
 
@@ -501,6 +568,327 @@ QTime DvbEpgFilter::bcdToTime(int bcd)
 		((bcd >> 4) & 0x0f) * 10 + (bcd & 0x0f));
 }
 
+static const QByteArray contentStr[16][16] = {
+	[0] = {},
+	[1] = {
+			/* Movie/Drama */
+		{},
+		{I18N_NOOP("Detective")},
+		{I18N_NOOP("Adventure")},
+		{I18N_NOOP("Science Fiction")},
+		{I18N_NOOP("Comedy")},
+		{I18N_NOOP("Soap")},
+		{I18N_NOOP("Romance")},
+		{I18N_NOOP("Classical")},
+		{I18N_NOOP("Adult")},
+		{I18N_NOOP("User defined")},
+	},
+	[2] = {
+			/* News/Current affairs */
+		{},
+		{I18N_NOOP("Weather")},
+		{I18N_NOOP("Magazine")},
+		{I18N_NOOP("Documentary")},
+		{I18N_NOOP("Discussion")},
+		{I18N_NOOP("User Defined")},
+	},
+	[3] = {
+			/* Show/Game show */
+		{},
+		{I18N_NOOP("Quiz")},
+		{I18N_NOOP("Variety")},
+		{I18N_NOOP("Talk")},
+		{I18N_NOOP("User Defined")},
+	},
+	[4] = {
+			/* Sports */
+		{},
+		{I18N_NOOP("Events")},
+		{I18N_NOOP("Magazine")},
+		{I18N_NOOP("Football")},
+		{I18N_NOOP("Tennis")},
+		{I18N_NOOP("Team")},
+		{I18N_NOOP("Athletics")},
+		{I18N_NOOP("Motor")},
+		{I18N_NOOP("Water")},
+		{I18N_NOOP("Winter")},
+		{I18N_NOOP("Equestrian")},
+		{I18N_NOOP("Martial")},
+		{I18N_NOOP("User Defined")},
+	},
+	[5] = {
+			/* Children's/Youth */
+		{},
+		{I18N_NOOP("Preschool")},
+		{I18N_NOOP("06 to 14")},
+		{I18N_NOOP("10 to 16")},
+		{I18N_NOOP("Educational")},
+		{I18N_NOOP("Cartoons")},
+		{I18N_NOOP("User Defined")},
+	},
+	[6] = {
+			/* Music/Ballet/Dance */
+		{},
+		{I18N_NOOP("Poprock")},
+		{I18N_NOOP("Classical")},
+		{I18N_NOOP("Folk")},
+		{I18N_NOOP("Jazz")},
+		{I18N_NOOP("Opera")},
+		{I18N_NOOP("Ballet")},
+		{I18N_NOOP("User Defined")},
+	},
+	[7] = {
+			/* Arts/Culture */
+		{},
+		{I18N_NOOP("Performance")},
+		{I18N_NOOP("Fine Arts")},
+		{I18N_NOOP("Religion")},
+		{I18N_NOOP("Traditional")},
+		{I18N_NOOP("Literature")},
+		{I18N_NOOP("Cinema")},
+		{I18N_NOOP("Experimental")},
+		{I18N_NOOP("Press")},
+		{I18N_NOOP("New Media")},
+		{I18N_NOOP("Magazine")},
+		{I18N_NOOP("Fashion")},
+		{I18N_NOOP("User Defined")},
+	},
+	[8] = {
+			/* Social/Political/Economics */
+		{},
+		{I18N_NOOP("Magazine")},
+		{I18N_NOOP("Advisory")},
+		{I18N_NOOP("People")},
+		{I18N_NOOP("User Defined")},
+	},
+	[9] = {
+			/* Education/Science/Factual */
+		{},
+		{I18N_NOOP("Nature")},
+		{I18N_NOOP("Technology")},
+		{I18N_NOOP("Medicine")},
+		{I18N_NOOP("Foreign")},
+		{I18N_NOOP("Social")},
+		{I18N_NOOP("Further")},
+		{I18N_NOOP("Language")},
+		{I18N_NOOP("User Defined")},
+	},
+	[10] = {
+			/* Leisure/Hobbies */
+		{},
+		{I18N_NOOP("Travel")},
+		{I18N_NOOP("Handicraft")},
+		{I18N_NOOP("Motoring")},
+		{I18N_NOOP("Fitness")},
+		{I18N_NOOP("Cooking")},
+		{I18N_NOOP("Shopping")},
+		{I18N_NOOP("Gardening")},
+		{I18N_NOOP("User Defined")},
+	},
+	[11] = {
+			/* Special characteristics */
+		{I18N_NOOP("Original Language")},
+		{I18N_NOOP("Black and White ")},
+		{I18N_NOOP("Unpublished")},
+		{I18N_NOOP("Live")},
+		{I18N_NOOP("Planostereoscopic")},
+		{I18N_NOOP("User Defined")},
+		{I18N_NOOP("User Defined 1")},
+		{I18N_NOOP("User Defined 2")},
+		{I18N_NOOP("User Defined 3")},
+		{I18N_NOOP("User Defined 4")}
+	}
+};
+
+static const QByteArray nibble1Str[16] = {
+	[0]  = {I18N_NOOP("Undefined")},
+	[1]  = {I18N_NOOP("Movie")},
+	[2]  = {I18N_NOOP("News")},
+	[3]  = {I18N_NOOP("Show")},
+	[4]  = {I18N_NOOP("Sports")},
+	[5]  = {I18N_NOOP("Children")},
+	[6]  = {I18N_NOOP("Music")},
+	[7]  = {I18N_NOOP("Culture")},
+	[8]  = {I18N_NOOP("Social")},
+	[9]  = {I18N_NOOP("Education")},
+	[10] = {I18N_NOOP("Leisure")},
+	[11] = {I18N_NOOP("Special")},
+	[12] = {I18N_NOOP("Reserved")},
+	[13] = {I18N_NOOP("Reserved")},
+	[14] = {I18N_NOOP("Reserved")},
+	[15] = {I18N_NOOP("User defined")},
+};
+
+static const QByteArray braNibble1Str[16] = {
+	[0]  = {I18N_NOOP("News")},
+	[1]  = {I18N_NOOP("Sports")},
+	[2]  = {I18N_NOOP("Education")},
+	[3]  = {I18N_NOOP("Soap opera")},
+	[4]  = {I18N_NOOP("Mini-series")},
+	[5]  = {I18N_NOOP("Series")},
+	[6]  = {I18N_NOOP("Variety")},
+	[7]  = {I18N_NOOP("Reality show")},
+	[8]  = {I18N_NOOP("Information")},
+	[9]  = {I18N_NOOP("Comical")},
+	[10] = {I18N_NOOP("Children")},
+	[11] = {I18N_NOOP("Erotic")},
+	[12] = {I18N_NOOP("Movie")},
+	[13] = {I18N_NOOP("Raffle, television sales, prizing")},
+	[14] = {I18N_NOOP("Debate/interview")},
+	[15] = {I18N_NOOP("Other")},
+};
+
+// Using the terms from the English version of NBR 15603-2:2007
+// The table omits nibble2="Other", as it is better to show nibble 1
+// definition instead.
+// when nibble2[x][0] == nibble1[x] and it has no other definition,
+// except for "Other", the field will be kept in blank, as the logic
+// will fall back to the definition at nibble 1.
+static QByteArray braNibble2Str[16][16] = {
+	[0] = {
+		{I18N_NOOP("News")},
+		{I18N_NOOP("Report")},
+		{I18N_NOOP("Documentary")},
+		{I18N_NOOP("Biography")},
+	},
+	[1] = {},
+	[2] = {
+		{I18N_NOOP("Educative")},
+	},
+	[3] = {},
+	[4] = {},
+	[5] = {},
+	[6] = {
+		{I18N_NOOP("Auditorium")},
+		{I18N_NOOP("Show")},
+		{I18N_NOOP("Musical")},
+		{I18N_NOOP("Making of")},
+		{I18N_NOOP("Feminine")},
+		{I18N_NOOP("Game show")},
+	},
+	[7] = {},
+	[8] = {
+		{I18N_NOOP("Cooking")},
+		{I18N_NOOP("Fashion")},
+		{I18N_NOOP("Country")},
+		{I18N_NOOP("Health")},
+		{I18N_NOOP("Travel")},
+	},
+	[9] = {},
+	[10] = {},
+	[11] = {},
+	[12] = {},
+	[13] = {
+		{I18N_NOOP("Raffle")},
+		{I18N_NOOP("Television sales")},
+		{I18N_NOOP("Prizing")},
+	},
+	[14] = {
+		{I18N_NOOP("Discussion")},
+		{I18N_NOOP("Interview")},
+	},
+	[15] = {
+		{I18N_NOOP("Adult cartoon")},
+		{I18N_NOOP("Interactive")},
+		{I18N_NOOP("Policy")},
+		{I18N_NOOP("Religion")},
+	},
+};
+
+QString DvbEpgFilter::getContent(DvbContentDescriptor &descriptor)
+{
+	QString content;
+
+	for (DvbEitContentEntry entry = descriptor.contents(); entry.isValid(); entry.advance()) {
+		const int nibble1 = entry.contentNibbleLevel1();
+		const int nibble2 = entry.contentNibbleLevel2();
+		QByteArray s;
+
+		// FIXME: should do it only for ISDB-Tb (Brazilian variation),
+		// as the Japanese variation uses the same codes as DVB
+		if (transponder.getTransmissionType() == DvbTransponderBase::IsdbT) {
+			s = braNibble2Str[nibble1][nibble2];
+			if (s == "")
+				s = braNibble1Str[nibble1];
+			if (s != "")
+				content += i18n(s) + "\n";
+		} else {
+			s = contentStr[nibble1][nibble2];
+			if (s == "")
+				s = nibble1Str[nibble1];
+			if (s != "")
+				content += i18n(s) + "\n";
+		}
+	}
+
+	if (content != "") {
+		// xgettext:no-c-format
+		return (i18n("Genre: %1", content));
+	}
+	return content;
+}
+
+/* As defined at ABNT NBR 15603-2 */
+static const QByteArray braRating[] = {
+	[0] = {I18N_NOOP("reserved")},
+	[1] = {I18N_NOOP("all audiences")},
+	[2] = {I18N_NOOP("10 years")},
+	[3] = {I18N_NOOP("12 years")},
+	[4] = {I18N_NOOP("14 years")},
+	[5] = {I18N_NOOP("16 years")},
+	[6] = {I18N_NOOP("18 years")},
+};
+
+#define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))
+
+QString DvbEpgFilter::getParental(DvbParentalRatingDescriptor &descriptor)
+{
+	QString parental;
+
+	for (DvbParentalRatingEntry entry = descriptor.contents(); entry.isValid(); entry.advance()) {
+		QString code;
+		code.append(QChar(entry.languageCode1()));
+		code.append(QChar(entry.languageCode2()));
+		code.append(QChar(entry.languageCode3()));
+
+		// Rating from 0x10 to 0xff are broadcaster's specific
+		if (entry.rating() == 0) {
+			// xgettext:no-c-format
+			parental += i18n("Country %1: not rated\n", code, entry.rating() + 3);
+		} else if (entry.rating() < 0x10) {
+			if (code == "BRA" && transponder.getTransmissionType() == DvbTransponderBase::IsdbT) {
+				unsigned int rating = entry.rating();
+
+				if (rating >= ARRAY_SIZE(braRating))
+					rating = 0;	// Reserved
+
+				QString GenStr;
+				int genre = entry.rating() >> 4;
+
+				if (genre & 0x2)
+					GenStr = i18n("violence / ");
+				if (genre & 0x4)
+					GenStr = i18n("sex / ");
+				if (genre & 0x1)
+					GenStr = i18n("drugs / ");
+				if (genre) {
+					GenStr.truncate(GenStr.size() - 2);
+					GenStr = " (" + GenStr + ")";
+				}
+
+				QString ratingStr = i18n(braRating[entry.rating()]);
+				// xgettext:no-c-format
+				parental += i18n("Country %1: rating: %2%3\n", code, ratingStr, GenStr);
+			} else {
+				// xgettext:no-c-format
+				parental += i18n("Country %1: rating: %2 years.\n", code, entry.rating() + 3);
+			}
+		}
+	}
+
+	return parental;
+}
+
 void DvbEpgFilter::processSection(const char *data, int size)
 {
 	unsigned char tableId = data[0];
@@ -512,6 +900,7 @@ void DvbEpgFilter::processSection(const char *data, int size)
 	DvbEitSection eitSection(data, size);
 
 	if (!eitSection.isValid()) {
+		qDebug("EPG section is invalid");
 		return;
 	}
 
@@ -529,11 +918,25 @@ void DvbEpgFilter::processSection(const char *data, int size)
 	}
 
 	if (!channel.isValid()) {
+		qDebug("EPG channel invalid");
 		return;
 	}
 
+	if (eitSection.entries().getLength())
+		qDebug("EPG table 0x%02x, extension 0x%04x, session %d/%d, size %d", eitSection.tableId(), eitSection.tableIdExtension(), eitSection.sectionNumber(), eitSection.lastSectionNumber(), eitSection.entries().getLength());
+
 	for (DvbEitSectionEntry entry = eitSection.entries(); entry.isValid(); entry.advance()) {
 		DvbEpgEntry epgEntry;
+
+		if (tableId == 0x4e)
+			epgEntry.type = DvbEpgEntry::EitActualTsPresentFollowing;
+		else if (tableId == 0x4f)
+			epgEntry.type = DvbEpgEntry::EitOtherTsPresentFollowing;
+		else if (tableId < 0x60)
+			epgEntry.type = DvbEpgEntry::EitActualTsSchedule;
+		else
+			epgEntry.type = DvbEpgEntry::EitOtherTsSchedule;
+
 		epgEntry.channel = channel;
 		epgEntry.begin = QDateTime(QDate::fromJulianDay(entry.startDate() + 2400001),
 			bcdToTime(entry.startTime()), Qt::UTC);
@@ -549,8 +952,8 @@ void DvbEpgFilter::processSection(const char *data, int size)
 					break;
 				}
 
-				epgEntry.title = eventDescriptor.eventName();
-				epgEntry.subheading = eventDescriptor.text();
+				epgEntry.title += eventDescriptor.eventName();
+				epgEntry.subheading += eventDescriptor.text();
 				break;
 			    }
 			case 0x4e: {
@@ -563,6 +966,26 @@ void DvbEpgFilter::processSection(const char *data, int size)
 				epgEntry.details += eventDescriptor.text();
 				break;
 			    }
+			case 0x54: {
+				DvbContentDescriptor eventDescriptor(descriptor);
+
+				if (!eventDescriptor.isValid()) {
+					break;
+				}
+
+				epgEntry.content += getContent(eventDescriptor);
+				break;
+			    }
+			case 0x55: {
+				DvbParentalRatingDescriptor eventDescriptor(descriptor);
+
+				if (!eventDescriptor.isValid()) {
+					break;
+				}
+
+				epgEntry.parental += getParental(eventDescriptor);
+				break;
+			    }
 			}
 		}
 
@@ -627,8 +1050,11 @@ void AtscEpgFilter::processMgtSection(const char *data, int size)
 	QList<int> newEitPids;
 	QList<int> newEttPids;
 
-	for (AtscMgtSectionEntry entry = mgtSection.entries(); (entryCount > 0) && entry.isValid();
-	     --entryCount, entry.advance()) {
+	AtscMgtSectionEntry entry = mgtSection.entries();
+	for (int i = 0; i < entryCount; i++) {
+		if (!entry.isValid())
+			break;
+
 		int tableType = entry.tableType();
 
 		if ((tableType >= 0x0100) && (tableType <= 0x017f)) {
@@ -648,6 +1074,8 @@ void AtscEpgFilter::processMgtSection(const char *data, int size)
 				newEttPids.insert(index, pid);
 			}
 		}
+		if (i < entryCount - 1)
+			entry.advance();
 	}
 
 	for (int i = 0; i < eitPids.size(); ++i) {
@@ -700,6 +1128,7 @@ void AtscEpgFilter::processEitSection(const char *data, int size)
 	AtscEitSection eitSection(data, size);
 
 	if (!eitSection.isValid()) {
+		qDebug("EPG section is invalid");
 		return;
 	}
 
@@ -710,32 +1139,33 @@ void AtscEpgFilter::processEitSection(const char *data, int size)
 	DvbSharedChannel channel = channelModel->findChannelById(fakeChannel);
 
 	if (!channel.isValid()) {
+		qDebug("channel is invalid");
 		return;
 	}
 
+	qDebug("Processing EIT section with size %d", size);
+
 	int entryCount = eitSection.entryCount();
 	// 1980-01-06T000000 minus 15 secs (= UTC - GPS in 2011)
 	QDateTime baseDateTime = QDateTime(QDate(1980, 1, 5), QTime(23, 59, 45), Qt::UTC);
 
-	for (AtscEitSectionEntry eitEntry = eitSection.entries();
-	     (entryCount > 0) && eitEntry.isValid(); --entryCount, eitEntry.advance()) {
+	AtscEitSectionEntry eitEntry = eitSection.entries();
+	for (int i = 0; i < entryCount; i++) {
+		if (!eitEntry.isValid())
+			break;
 		DvbEpgEntry epgEntry;
 		epgEntry.channel = channel;
 		epgEntry.begin = baseDateTime.addSecs(eitEntry.startTime());
-		epgEntry.duration = QTime().addSecs(eitEntry.duration());
+		epgEntry.duration = QTime(0, 0, 0).addSecs(eitEntry.duration());
 		epgEntry.title = eitEntry.title();
 
 		quint32 id = ((quint32(fakeChannel.networkId) << 16) | quint32(eitEntry.eventId()));
 		DvbSharedEpgEntry entry = epgEntries.value(id);
 
-		if (entry.isValid() && (entry->channel == epgEntry.channel) &&
-		    (entry->begin == epgEntry.begin) && (entry->duration == epgEntry.duration) &&
-		    (entry->title == epgEntry.title)) {
-			continue;
-		}
-
 		entry = epgModel->addEntry(epgEntry);
 		epgEntries.insert(id, entry);
+		if ( i < entryCount -1)
+			eitEntry.advance();
 	}
 }
 
