@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <KLocalizedString>
 #include <QDebug>
 #if QT_VERSION < 0x050500
 # define qInfo qDebug
@@ -25,7 +26,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <KLocalizedString>
 #include <KMessageBox>
 #include <QDir>
 #include <QLocale>
@@ -129,7 +129,7 @@ QPixmap DvbOsd::paintOsd(QRect &rect, const QFont &font, Qt::LayoutDirection)
 					firstEntry.subheading, &boundingRect);
 			}
 
-			if (!firstEntry.details.isEmpty()) {
+			if (!firstEntry.details.isEmpty() && firstEntry.details != firstEntry.title) {
 				painter.drawText(entryRect.x(), boundingRect.bottom() + 1,
 					entryRect.width(),
 					rect.height() - boundingRect.bottom() - 1,
@@ -451,16 +451,14 @@ void DvbLiveView::playbackStatusChanged(MediaWidget::PlaybackStatus playbackStat
 
 		if (internal->timeShiftFile.exists() ||
 		    !internal->timeShiftFile.open(QIODevice::WriteOnly)) {
-			qInfo() << "DvbLiveView::playbackStatusChanged: cannot open file" <<
-				internal->timeShiftFile.fileName();
+			qWarning("Cannot open file %s", qPrintable(internal->timeShiftFile.fileName()));
 			internal->timeShiftFile.setFileName(QDir::homePath() + QLatin1String("/TimeShift-") +
 				QDateTime::currentDateTime().toString(QLatin1String("yyyyMMddThhmmss")) +
 				QLatin1String(".m2t"));
 
 			if (internal->timeShiftFile.exists() ||
 			    !internal->timeShiftFile.open(QIODevice::WriteOnly)) {
-				qInfo() << "DvbLiveView::playbackStatusChanged: cannot open file" <<
-					internal->timeShiftFile.fileName();
+				qWarning("Cannot open file %s", qPrintable(internal->timeShiftFile.fileName()));
 				mediaWidget->stop();
 				break;
 			}
@@ -595,21 +593,21 @@ DvbLiveViewInternal::DvbLiveViewInternal(QObject *parent) : QObject(parent), med
 	updateUrl();
 
 	if (mkfifo(QFile::encodeName(fileName).constData(), 0600) != 0) {
-		qInfo() << "DvbLiveViewInternal::DvbLiveViewInternal: mkfifo failed";
+		qWarning("Failed to open a fifo. Error: %d", errno);
 		return;
 	}
 
 	readFd = open(QFile::encodeName(fileName).constData(), O_RDONLY | O_NONBLOCK);
 
 	if (readFd < 0) {
-		qInfo() << "DvbLiveViewInternal::DvbLiveViewInternal: open failed";
+		qWarning("Failed to open fifo for read. Error: %d", errno);
 		return;
 	}
 
 	writeFd = open(QFile::encodeName(fileName).constData(), O_WRONLY | O_NONBLOCK);
 
 	if (writeFd < 0) {
-		qInfo() << "DvbLiveViewInternal::DvbLiveViewInternal: open failed";
+		qWarning("Failed to open fifo for write. Error: %d", errno);
 		return;
 	}
 
