@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <KLocalizedString>
 #include <QDebug>
 #if QT_VERSION < 0x050500
 # define qInfo qDebug
@@ -25,13 +26,13 @@
 
 #include <KActionCollection>
 #include <KConfigGroup>
-#include <KLocalizedString>
 #include <KSharedConfig>
 #include <KToolBar>
 #include <QBoxLayout>
 #include <QComboBox>
 #include <QContextMenuEvent>
 #include <QDBusInterface>
+#include <QFileDialog>
 #include <QLabel>
 #include <QMenu>
 #include <QMimeData>
@@ -81,7 +82,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	osdWidget = new OsdWidget(this);
 
 	actionPrevious = new QWidgetAction(this);
-	actionPrevious->setIcon(QIcon::fromTheme(QLatin1String("media-skip-backward")));
+	actionPrevious->setIcon(QIcon::fromTheme(QLatin1String("media-skip-backward"), QIcon(":media-skip-backward")));
 	actionPrevious->setText(i18n("Previous"));
 	actionPrevious->setShortcut(QKeySequence(Qt::Key_PageUp, Qt::Key_MediaPrevious));
 	connect(actionPrevious, SIGNAL(triggered()), this, SLOT(previous()));
@@ -92,14 +93,14 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	actionPlayPause->setShortcut(QKeySequence(Qt::Key_Space, Qt::Key_MediaPlay));
 	textPlay = i18n("Play");
 	textPause = i18n("Pause");
-	iconPlay = QIcon::fromTheme(QLatin1String("media-playback-start"));
-	iconPause = QIcon::fromTheme(QLatin1String("media-playback-pause"));
+	iconPlay = QIcon::fromTheme(QLatin1String("media-playback-start"), QIcon(":media-playback-start"));
+	iconPause = QIcon::fromTheme(QLatin1String("media-playback-pause"), QIcon(":media-playback-pause"));
 	connect(actionPlayPause, SIGNAL(triggered(bool)), this, SLOT(pausedChanged(bool)));
 	toolBar->addAction(collection->addAction(QLatin1String("controls_play_pause"), actionPlayPause));
 	menu->addAction(actionPlayPause);
 
 	actionStop = new QWidgetAction(this);
-	actionStop->setIcon(QIcon::fromTheme(QLatin1String("media-playback-stop")));
+	actionStop->setIcon(QIcon::fromTheme(QLatin1String("media-playback-stop"), QIcon(":media-playback-stop")));
 	actionStop->setText(i18n("Stop"));
 	actionStop->setShortcut(QKeySequence(Qt::Key_Backspace, Qt::Key_MediaStop));
 	connect(actionStop, SIGNAL(triggered()), this, SLOT(stop()));
@@ -107,7 +108,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	menu->addAction(actionStop);
 
 	actionNext = new QWidgetAction(this);
-	actionNext->setIcon(QIcon::fromTheme(QLatin1String("media-skip-forward")));
+	actionNext->setIcon(QIcon::fromTheme(QLatin1String("media-skip-forward"), QIcon(":media-skip-forward")));
 	actionNext->setText(i18n("Next"));
 	actionNext->setShortcut(QKeySequence(Qt::Key_PageDown, Qt::Key_MediaNext));
 	connect(actionNext, SIGNAL(triggered()), this, SLOT(next()));
@@ -116,14 +117,14 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	menu->addSeparator();
 
 	fullScreenAction = new QWidgetAction(this);
-	fullScreenAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen")));
+	fullScreenAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen"), QIcon(":view-fullscreen")));
 	fullScreenAction->setText(i18nc("'Playback' menu", "Full Screen Mode"));
 	fullScreenAction->setShortcut(Qt::Key_F);
 	connect(fullScreenAction, SIGNAL(triggered()), this, SLOT(toggleFullScreen()));
 	menu->addAction(collection->addAction(QLatin1String("view_fullscreen"), fullScreenAction));
 
 	minimalModeAction = new QWidgetAction(this);
-	minimalModeAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen")));
+	minimalModeAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen"), QIcon(":view-fullscreen")));
 	minimalModeAction->setText(i18nc("'Playback' menu", "Minimal Mode"));
 	minimalModeAction->setShortcut(Qt::Key_Period);
 	connect(minimalModeAction, SIGNAL(triggered()), this, SLOT(toggleMinimalMode()));
@@ -137,26 +138,44 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	audioStreamModel = new QStringListModel(toolBar);
 	audioStreamBox->setModel(audioStreamModel);
 
-	subtitleBox = new QComboBox(toolBar);
+	QMenu *subtitleMenu = new QMenu(i18nc("'Subtitle' menu", "Subtitle"), this);
+	subtitleBox = new QComboBox(this);
+	QWidgetAction *action = new QWidgetAction(this);
+	action->setDefaultWidget(subtitleBox);
 	textSubtitlesOff = i18nc("subtitle selection entry", "off");
 	connect(subtitleBox, SIGNAL(currentIndexChanged(int)),
 		this, SLOT(currentSubtitleChanged(int)));
-	toolBar->addWidget(subtitleBox);
-
 	subtitleModel = new QStringListModel(toolBar);
 	subtitleBox->setModel(subtitleModel);
+	subtitleMenu->addAction(action);
+	menu->addMenu(subtitleMenu);
+	action = new QWidgetAction(this);
+	action->setText(i18nc("'Subtitle' menu", "Add subtitle file"));
+	action->setIcon(QIcon::fromTheme(QLatin1String("application-x-subrip"), QIcon(":application-x-subrip")));
+	connect(action, &QWidgetAction::triggered, this, &MediaWidget::openSubtitle);
+	subtitleMenu->addAction(action);
+
+	menu->addMenu(subtitleMenu);
 
 	QMenu *audioMenu = new QMenu(i18nc("'Playback' menu", "Audio"), this);
+	action = new QWidgetAction(this);
+	action->setIcon(QIcon::fromTheme(QLatin1String("audio-card"), QIcon(":audio-card")));
+	action->setText(i18nc("'Audio' menu", "Audio Device"));
 
-	QWidgetAction *action = new QWidgetAction(this);
-	action->setIcon(QIcon::fromTheme(QLatin1String("audio-volume-high")));
+	audioDevMenu = new QMenu(i18nc("'Playback' menu", "Audio Device"), audioMenu);
+	action = new QWidgetAction(this);
+	connect(audioDevMenu, &QMenu::aboutToShow, this, &MediaWidget::getAudioDevices);
+	audioMenu->addMenu(audioDevMenu);
+
+	action = new QWidgetAction(this);
+	action->setIcon(QIcon::fromTheme(QLatin1String("audio-volume-high"), QIcon(":audio-volume-high")));
 	action->setText(i18nc("'Audio' menu", "Increase Volume"));
 	action->setShortcut(QKeySequence(Qt::Key_Plus, Qt::Key_VolumeUp));
 	connect(action, SIGNAL(triggered()), this, SLOT(increaseVolume()));
 	audioMenu->addAction(collection->addAction(QLatin1String("controls_increase_volume"), action));
 
 	action = new QWidgetAction(this);
-	action->setIcon(QIcon::fromTheme(QLatin1String("audio-volume-low")));
+	action->setIcon(QIcon::fromTheme(QLatin1String("audio-volume-low"), QIcon(":audio-volume-low")));
 	action->setText(i18nc("'Audio' menu", "Decrease Volume"));
 	action->setShortcut(QKeySequence(Qt::Key_Minus, Qt::Key_VolumeDown));
 	connect(action, SIGNAL(triggered()), this, SLOT(decreaseVolume()));
@@ -164,8 +183,8 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 
 	muteAction = new QWidgetAction(this);
 	muteAction->setText(i18nc("'Audio' menu", "Mute Volume"));
-	mutedIcon = QIcon::fromTheme(QLatin1String("audio-volume-muted"));
-	unmutedIcon = QIcon::fromTheme(QLatin1String("audio-volume-medium"));
+	mutedIcon = QIcon::fromTheme(QLatin1String("audio-volume-muted"), QIcon(":audio-volume-muted"));
+	unmutedIcon = QIcon::fromTheme(QLatin1String("audio-volume-medium"), QIcon(":audio-volume-medium"));
 	muteAction->setIcon(unmutedIcon);
 	muteAction->setShortcut(QKeySequence(Qt::Key_M, Qt::Key_VolumeMute));
 	connect(muteAction, SIGNAL(triggered()), this, SLOT(mutedChanged()));
@@ -178,7 +197,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	menu->addSeparator();
 
 	deinterlaceAction = new QWidgetAction(this);
-	deinterlaceAction->setIcon(QIcon::fromTheme(QLatin1String("format-justify-center")));
+	deinterlaceAction->setIcon(QIcon::fromTheme(QLatin1String("format-justify-center"), QIcon(":format-justify-center")));
 	deinterlaceAction->setText(i18nc("'Video' menu", "Deinterlace"));
 	deinterlaceAction->setCheckable(true);
 	deinterlaceAction->setChecked(
@@ -279,7 +298,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	toolBar->addAction(collection->addAction(QLatin1String("controls_volume_slider"), action));
 
 	jumpToPositionAction = new QWidgetAction(this);
-	jumpToPositionAction->setIcon(QIcon::fromTheme(QLatin1String("go-jump")));
+	jumpToPositionAction->setIcon(QIcon::fromTheme(QLatin1String("go-jump"), QIcon(":go-jump")));
 	jumpToPositionAction->setText(i18nc("@action:inmenu", "Jump to Position..."));
 	jumpToPositionAction->setShortcut(Qt::CTRL + Qt::Key_J);
 	connect(jumpToPositionAction, SIGNAL(triggered()), this, SLOT(jumpToPosition()));
@@ -297,7 +316,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 		this, SLOT(longSkipDurationChanged(int)));
 
 	longSkipBackwardAction = new QWidgetAction(this);
-	longSkipBackwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-backward")));
+	longSkipBackwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-backward"), QIcon(":media-skip-backward")));
 	// xgettext:no-c-format
 	longSkipBackwardAction->setText(i18nc("submenu of 'Skip'", "Skip %1s Backward", longSkipDuration));
 	longSkipBackwardAction->setShortcut(Qt::SHIFT + Qt::Key_Left);
@@ -306,7 +325,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 		collection->addAction(QLatin1String("controls_long_skip_backward"), longSkipBackwardAction));
 
 	shortSkipBackwardAction = new QWidgetAction(this);
-	shortSkipBackwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-backward")));
+	shortSkipBackwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-backward"), QIcon(":media-skip-backward")));
 	// xgettext:no-c-format
 	shortSkipBackwardAction->setText(i18nc("submenu of 'Skip'", "Skip %1s Backward", shortSkipDuration));
 	shortSkipBackwardAction->setShortcut(Qt::Key_Left);
@@ -315,7 +334,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 		collection->addAction(QLatin1String("controls_skip_backward"), shortSkipBackwardAction));
 
 	shortSkipForwardAction = new QWidgetAction(this);
-	shortSkipForwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-forward")));
+	shortSkipForwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-forward"), QIcon(":media-skip-forward")));
 	// xgettext:no-c-format
 	shortSkipForwardAction->setText(i18nc("submenu of 'Skip'", "Skip %1s Forward", shortSkipDuration));
 	shortSkipForwardAction->setShortcut(Qt::Key_Right);
@@ -324,7 +343,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 		collection->addAction(QLatin1String("controls_skip_forward"), shortSkipForwardAction));
 
 	longSkipForwardAction = new QWidgetAction(this);
-	longSkipForwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-forward")));
+	longSkipForwardAction->setIcon(QIcon::fromTheme(QLatin1String("media-skip-forward"), QIcon(":media-skip-forward")));
 	// xgettext:no-c-format
 	longSkipForwardAction->setText(i18nc("submenu of 'Skip'", "Skip %1s Forward", longSkipDuration));
 	longSkipForwardAction->setShortcut(Qt::SHIFT + Qt::Key_Right);
@@ -332,7 +351,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	navigationMenu->addAction(
 		collection->addAction(QLatin1String("controls_long_skip_forward"), longSkipForwardAction));
 
-	toolBar->addAction(QIcon::fromTheme(QLatin1String("player-time")), i18n("Seek Slider"))->setEnabled(false);
+	toolBar->addAction(QIcon::fromTheme(QLatin1String("player-time"), QIcon(":player-time")), i18n("Seek Slider"))->setEnabled(false);
 
 	action = new QWidgetAction(this);
 	action->setText(i18n("Seek Slider"));
@@ -345,7 +364,7 @@ MediaWidget::MediaWidget(QMenu *menu_, QToolBar *toolBar, KActionCollection *col
 	toolBar->addAction(collection->addAction(QLatin1String("controls_position_slider"), action));
 
 	menuAction = new QWidgetAction(this);
-	menuAction->setIcon(QIcon::fromTheme(QLatin1String("media-optical-video")));
+	menuAction->setIcon(QIcon::fromTheme(QLatin1String("media-optical-video"), QIcon(":media-optical-video")));
 	menuAction->setText(i18nc("dvd navigation", "DVD Menu"));
 	connect(menuAction, SIGNAL(triggered()), this, SLOT(toggleMenu()));
 	menu->addAction(collection->addAction(QLatin1String("controls_toggle_menu"), menuAction));
@@ -440,12 +459,12 @@ void MediaWidget::setDisplayMode(DisplayMode displayMode_)
 		switch (displayMode) {
 		case NormalMode:
 		case MinimalMode:
-			fullScreenAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen")));
+			fullScreenAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen"), QIcon(":view-fullscreen")));
 			fullScreenAction->setText(i18nc("'Playback' menu", "Full Screen Mode"));
 			break;
 		case FullScreenMode:
 		case FullScreenReturnToMinimalMode:
-			fullScreenAction->setIcon(QIcon::fromTheme(QLatin1String("view-restore")));
+			fullScreenAction->setIcon(QIcon::fromTheme(QLatin1String("view-restore"), QIcon(":view-restore")));
 			fullScreenAction->setText(i18nc("'Playback' menu",
 				"Exit Full Screen Mode"));
 			break;
@@ -455,11 +474,11 @@ void MediaWidget::setDisplayMode(DisplayMode displayMode_)
 		case NormalMode:
 		case FullScreenMode:
 		case FullScreenReturnToMinimalMode:
-			minimalModeAction->setIcon(QIcon::fromTheme(QLatin1String("view-restore")));
+			minimalModeAction->setIcon(QIcon::fromTheme(QLatin1String("view-restore"), QIcon(":view-restore")));
 			minimalModeAction->setText(i18nc("'Playback' menu", "Minimal Mode"));
 			break;
 		case MinimalMode:
-			minimalModeAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen")));
+			minimalModeAction->setIcon(QIcon::fromTheme(QLatin1String("view-fullscreen"), QIcon(":view-fullscreen")));
 			minimalModeAction->setText(i18nc("'Playback' menu", "Exit Minimal Mode"));
 			break;
 		}
@@ -490,6 +509,29 @@ void MediaWidget::mediaSourceDestroyed(MediaSource *mediaSource)
 	}
 }
 
+void MediaWidget::openSubtitle()
+{
+	QUrl url = QFileDialog::getOpenFileName(this, i18nc("@title:window", "Open subtitle"),".", i18n("Subtitles (*.cdg *.idx *.srt " \
+				"*.sub *.utf *.ass " \
+				"*.ssa *.aqt " \
+				"*.jss *.psb " \
+				"*.rt *.smi *.txt " \
+				"*.smil *.stl *.usf " \
+				"*.dks *.pjs *.mpl2 *.mks " \
+				"*.vtt *.ttml *.dfxp"));
+
+	setSubtitle(url);
+}
+
+void MediaWidget::setSubtitle(QUrl url)
+{
+	if (!url.isValid()) {
+		return;
+	}
+
+	backend->setExternalSubtitle(url);
+}
+
 void MediaWidget::play(const QUrl &url, const QUrl &subtitleUrl)
 {
 	// FIXME mem-leak
@@ -640,6 +682,11 @@ void MediaWidget::stop()
 	source->playbackStatusChanged(Idle);
 }
 
+void MediaWidget::setAudioCard()
+{
+	QAction *action = qobject_cast<QAction *>(sender());
+	backend->setAudioDevice(action->data().toString());
+}
 void MediaWidget::increaseVolume()
 {
 	// QSlider ensures that the value is within the range
@@ -766,7 +813,7 @@ void MediaWidget::aspectRatioChanged(QAction *action)
 		return;
 	}
 
-	qInfo() << "MediaWidget::aspectRatioChanged: internal error";
+	qWarning("Internal error");
 }
 
 void MediaWidget::autoResizeTriggered(QAction *action)
@@ -794,7 +841,7 @@ void MediaWidget::autoResizeTriggered(QAction *action)
 		}
 	}
 
-	qInfo() << "MediaWidget::autoResizeTriggered: internal error";
+	qWarning("Internal error");
 }
 
 void MediaWidget::pausedChanged(bool paused)
@@ -883,24 +930,17 @@ void MediaWidget::currentAudioStreamChanged(int currentAudioStream)
 
 void MediaWidget::currentSubtitleChanged(int currentSubtitle)
 {
-	if (!blockBackendUpdates) {
-		--currentSubtitle;
+	if (blockBackendUpdates)
+		return;
 
-		if (source->overrideSubtitles()) {
-			source->setCurrentSubtitle(currentSubtitle);
-			return;
-		}
-
-		source->setCurrentSubtitle(currentSubtitle - backend->getSubtitles().size());
-
-		if (currentSubtitle >= backend->getSubtitles().size()) {
-			currentSubtitle = -1;
-		}
-
-		if (backend->getCurrentSubtitle() != currentSubtitle) {
-			backend->setCurrentSubtitle(currentSubtitle);
-		}
+	if (source->overrideSubtitles()) {
+		source->setCurrentSubtitle(currentSubtitle - 1);
+		return;
 	}
+
+	source->setCurrentSubtitle(currentSubtitle - 1 - backend->getSubtitles().size());
+
+	backend->setCurrentSubtitle(currentSubtitle);
 }
 
 void MediaWidget::toggleMenu()
@@ -943,6 +983,21 @@ void MediaWidget::longSkipDurationChanged(int longSkipDuration)
 		longSkipDuration));
 }
 
+void MediaWidget::getAudioDevices()
+{
+	foreach(QAction *action, audioDevMenu->actions()) {
+		audioDevMenu->removeAction(action);
+	}
+
+	foreach(const QString &device, backend->getAudioDevices()) {
+		QAction *action = new QWidgetAction(this);
+		action->setText(device);
+		action->setData(device);
+		connect(action, SIGNAL(triggered()), this, SLOT(setAudioCard()));
+		audioDevMenu->addAction(action);
+	}
+}
+
 void MediaWidget::audioStreamsChanged()
 {
 	QStringList items;
@@ -970,7 +1025,7 @@ void MediaWidget::audioStreamsChanged()
 void MediaWidget::subtitlesChanged()
 {
 	QStringList items(textSubtitlesOff);
-	int currentIndex;
+	int currentIndex = 0;
 
 	if (source->overrideSubtitles()) {
 		items += source->getSubtitles();
@@ -989,7 +1044,7 @@ void MediaWidget::subtitlesChanged()
 	} else {
 		items += backend->getSubtitles();
 		items += source->getSubtitles();
-		currentIndex = (backend->getCurrentSubtitle() + 1);
+		currentIndex = (backend->getCurrentSubtitle());
 		int currentSourceIndex = source->getCurrentSubtitle();
 
 		if (currentSourceIndex >= 0) {
@@ -1003,6 +1058,9 @@ void MediaWidget::subtitlesChanged()
 		subtitleModel->setStringList(items);
 	}
 
+	if (currentIndex < 0)
+		currentIndex = 0;
+
 	subtitleBox->setCurrentIndex(currentIndex);
 	subtitleBox->setEnabled(items.size() > 1);
 	blockBackendUpdates = false;
@@ -1104,6 +1162,9 @@ void MediaWidget::wheelEvent(QWheelEvent *event)
 
 void MediaWidget::playbackFinished()
 {
+	if (source->getType() == MediaSource::Url)
+		emit playlistNext();
+
 	source->playbackFinished();
 }
 
