@@ -18,12 +18,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <KLocalizedString>
 #include <QDebug>
 #if QT_VERSION < 0x050500
 # define qInfo qDebug
 #endif
 
-#include <KLocalizedString>
 #include <QDir>
 #include <QDomDocument>
 #include <QLocale>
@@ -42,14 +42,14 @@ bool Playlist::load(const QUrl &url_, Format format)
 
 	if (localFile.isEmpty()) {
 		// FIXME
-		qInfo() << "Playlist::load: opening remote playlists not supported yet";
+		qInfo("Opening remote playlists not supported yet");
 		return false;
 	}
 
 	QFile file(localFile);
 
 	if (!file.open(QIODevice::ReadOnly)) {
-		qInfo() << "Playlist::load: cannot open file" << file.fileName();
+		qWarning("Cannot open file %s", qPrintable(file.fileName()));
 		return false;
 	}
 
@@ -75,14 +75,14 @@ bool Playlist::save(Format format) const
 
 	if (localFile.isEmpty()) {
 		// FIXME
-		qInfo() << "Playlist::save: opening remote playlists not supported yet";
+		qInfo("Opening remote playlists not supported yet");
 		return false;
 	}
 
 	QFile file(localFile);
 
 	if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-		qInfo() << "Playlist::save: cannot open file" << file.fileName();
+		qWarning("Cannot open file %s", qPrintable(file.fileName()));
 		return false;
 	}
 
@@ -762,7 +762,7 @@ QVariant PlaylistModel::data(const QModelIndex &index, int role) const
 {
 	if (role == Qt::DecorationRole) {
 		if ((index.row() == visiblePlaylist->currentTrack) && (index.column() == 0)) {
-			return QIcon::fromTheme(QLatin1String("arrow-right"));
+			return QIcon::fromTheme(QLatin1String("arrow-right"), QIcon(":arrow-right"));
 		}
 	} else if (role == Qt::DisplayRole) {
 		switch (index.column()) {
